/*********************************************************/
/* This class controls access to the large colour window */
/*                                                       */
/*         (c)1994 Alexis 'Milamber' Ashley              */
/*********************************************************/
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __DBL_WINDOW__
#define __DBL_WINDOW__

#include "yuv_window.H"

#ifdef SH_MEM
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#endif

#define LUM_RANGE 8
#define CR_RANGE 4
#define CB_RANGE 4

class DblWindow : public YUV_Window
{
public:
  DblWindow(UI_Globals *parent,world_c x,world_c y,unsigned int height,
	    unsigned int width, void (*callback)(void));
  ~DblWindow(void);
  status DisplayFrame(frame &, bool nicely=true);
private:
  unsigned char col_array[256];
  int *lum_values;
  int *cr_values;
  int *cb_values;
  unsigned char *dith_a; // This array contains the actual pixel values for
                         // each possible 2x2 dither pattern
  int lval_a[256];       // Arrays for mapping lum, cr and cb
  int rval_a[256];       // values to portions of the dither
  int bval_a[256];       // pattern.
  XImage *ximage;
  GC gc;
  bool image_available;
#ifdef SH_MEM
  bool using_shm;                        /* True if shared memory in use */
  int CompletionType;                    /* ???? */
  XShmSegmentInfo shminfo;               /* Segment info.  */
#endif
  void (*callback)(void);
/* **** Private functions **** */
  DblWindow(const DblWindow &);      //Stop anyone using the copy constructor
  status event_handler(XEvent *);    // The (expose) event handler
  status InitColours(void);          // Initialises the colour arrays
  status InitDitherArray(void);      // Initialises the dither arrays
  void DitherImage(unsigned char *lum,unsigned char *cr,unsigned char *cb,
		   unsigned char *out,int w,int h);
};

#endif

