/*******************************************************/
/* This class controls access to a small colour window */
/*                                                     */
/*         (c)1995 Alexis 'Milamber' Ashley            */
/*******************************************************/
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __COLOUR_WINDOW__
#define __COLOUR_WINDOW__

#include "yuv_window.H"

#ifdef SH_MEM
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#endif

#define LUM_RANGE 8
#define CR_RANGE 4
#define CB_RANGE 4
#define DITH_SIZE 16

class ColourWindow : public YUV_Window
{
public:
  ColourWindow(UI_Globals *parent,world_c x,world_c y,unsigned int height,
	     unsigned int width, void (*callback)(void) );
  ~ColourWindow(void);
  status DisplayFrame(frame &, bool nicely=true);
private:
  unsigned char col_array[256];       // This holds all the colours allocated
                                      // from X
  int *lum_values;
  int *cr_values;
  int *cb_values;
  unsigned char *dith_a; // This array contains the actual pixel values for
                         // each possible 2x2 dither pattern
  int lval_a[256];       // Arrays for mapping lum, cr and cb
  int rval_a[256];       // values to portions of the dither
  int bval_a[256];       // pattern.
  unsigned char l_darrays[DITH_SIZE][256];
  unsigned char cr_darrays[DITH_SIZE][256];
  unsigned char cb_darrays[DITH_SIZE][256];
  XImage *ximage;                     // This stores the XImage, which is a
                                      // structure used to send a pixmap to X
  GC gc;                              // Our gc for this window
  bool image_available;
  void (*callback)(void);
#ifdef SH_MEM
  bool using_shm;                        /* True if shared memory in use */
  int CompletionType;                    /* ???? */
  XShmSegmentInfo shminfo;               /* Segment info.  */
#endif
/* **** Private functions **** */
  ColourWindow(const ColourWindow &); //Stop anyone using the copy constructor
  status InitColours(void);           //Initialises the colours
  status InitDitherArray(void);       //Initialises the dither array
  status event_handler(XEvent *);     //The (expose) event handler
  void ColourWindow::DitherImage(unsigned char *lum,unsigned char *cr,
				 unsigned char *cb,unsigned char *out,
				 int w,int h);
};

#endif

