/* This is the header file for time display windows of my editor class*/
/* (c)1994 Alexis 'Milamber' Ashley */
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __TIME_WIN__
#define __TIME_WIN__

#include "sub_win.H"
#include "codec.H"

class TimeWindow : public SimpleSubWindow
{
protected:
  static char DefaultFont[80];   // Default font used for sub windows
  static unsigned int BorderWidth;      // Size of border 
public:
  TimeWindow(UI_Globals *parent,world_c x,world_c y,
	     corner relative_to=topleft,
	     const char * const fgcolour = Foreground,
	     const char * const bgcolour = Background,
	     const char * const font = DefaultFont,
	     unsigned int Border_Width=BorderWidth);
  ~TimeWindow(void);
  void SetTime(Codec::hms_addr value, bool nicely);
private:
// *** Private vars ***
  GC GraphicsContext;      // The GC for the window
  unsigned int xpos;       // X position of the text
  unsigned int ypos;       // Y position of the text
  char string[20];         // Stores the time string "00h 00m 00.00s"
  XFontStruct *xfs;        // The font for this subwindow
  XSegment high_seg[4];    // Stores coords of the bright lines to draw
  XSegment low_seg[4];     // Stores coords of the dark lines to draw
  int SendEventFlag;       // Used to stop multiple send events being made
// *** Static private vars
  static XFontStruct *Default_Font; // Default font for all time windows
  static char high_name[80];        // Default colour for bright 3D lines
  static char low_name[80];         // Default colour for dark 3D lines
  static opt_list resources[];      // Resource list for all time windows
  static colour high_colour;        // Colour value for bright lines
  static colour low_colour;         // Colour value for dark lines
  static GC highlight;              // GC for bright lines
  static GC lowlight;               // GC for dark lines
  static int count;                 // Count of how many time windows there are
// *** Private functions
  status event_handler(XEvent *);  // The subwindow's event handler
  unsigned int calc_height(const char * const font);
  unsigned int calc_width(const char * const font);
};

#endif


