/**********************************************************/
/* This class controls access to a tiny monochrome window */
/*                                                        */
/*         (c)1994 Alexis 'Milamber' Ashley               */
/**********************************************************/
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __TINY_WINDOW__
#define __TINY_WINDOW__

#include "yuv_window.H"

#ifdef SH_MEM
#include <sys/ipc.h>
#include <sys/shm.h>
#include <X11/extensions/XShm.h>
#endif

#define GRAY_RANGE 32

class TinyWindow : public YUV_Window
{
public:
  TinyWindow(UI_Globals *parent,world_c x,world_c y,unsigned int height,
	     unsigned int width, void (*callback)(void) );
  ~TinyWindow(void);
  status DisplayFrame(frame &, bool nicely=true);
private:
  unsigned char col_array[GRAY_RANGE]; // This holds all the colours allocated
                                      // from X
  unsigned char dith_array[256];      // This array maps each byte unto a
                                      // colour from col_array;
  XImage *ximage;                     // This stores the XImage, which is a
                                      // structure used to send a pixmap to X
  GC gc;                              // Our gc for this window
  bool image_available;
  void (*callback)(void);
#ifdef SH_MEM
  bool using_shm;                        /* True if shared memory in use */
  int CompletionType;                    /* ???? */
  XShmSegmentInfo shminfo;               /* Segment info.  */
#endif
/* **** Private functions **** */
  TinyWindow(const TinyWindow &);     //Stop anyone using the copy constructor
  status InitColours(void);           //Initialises the colours
  status InitDitherArray(void);       //Initialises the dither array
  status event_handler(XEvent *);     //The (expose) event handler
  void DitherImage(unsigned char *lum,unsigned char *out,int height,int width);
};

#endif

