/* LIST.H     Header file for class List     (c)1993 A Ashley */
/* Edited to use double linked list 1994  (c)1994 Alexis Ashley */

#ifndef __LIST__

#define __LIST__

#include <assert.h>
#include "typedefs.H"

#ifndef NULL
#define NULL 0
#endif

template <class eltype>
class List
{
private:
  struct linked_list
    {
      eltype data;              // This is where the data is stored
      linked_list *next;        // This points to the next element
                                // in the list
      linked_list *prior;       // This points to the previous element
                                // in the list
    };

  struct linked_list *cur_pos;          // This stores the current position
                                        // in the list
	    
  struct linked_list *prev_pos;         // This stores the previous position
                                        // in the list

  struct linked_list *list_start;       // This stores the pointer to the 
                                        // start of the list

  long cur_pos_num;                     // This holds the number returned
		                           // by the get_pos function

 protected:

 public:

/* ********** The Constructor Function ************ */

  List(void)
    {
      list_start = NULL;          // Set list to be empty
      cur_pos = NULL;
      prev_pos = NULL;
      cur_pos_num = 0;
    }

/* ********** The Destructor Function ************ */

  ~List(void)
    {
      Rewind();
      if(cur_pos!=NULL)         // There's nothing to do if the list is empty.
	{
	  while(Advance()==true)
	    ;
	  while(Previous()==true)
	    {
	      delete cur_pos->next;
	      cur_pos->next = NULL;
	    }
	  delete cur_pos;
	}
    }

/* ********** void Rewind(void) ******************
   This will set the current position to the start
   of the list.                                       */

  void Rewind(void)         // Set cur_pos to point to the start of the list
    {
      cur_pos = list_start;
      prev_pos = NULL;
      cur_pos_num = 0;
    }
	    
/* ********** eltype Read(void) *******************
   This will return the current element in the list,
   or NULL if the list is empty, or if we are at the
   end of the list.                                    */

  eltype Read(void) const
    {
      assert(cur_pos!=NULL);
      return(cur_pos->data);
    }

/* ********** void Write(const eltype w_data) ****************
   Write w_data to the current position in the list.  If
   the list is empty, the first element is created to
   store this data.                                        */

  void Write(const eltype w_data)
    {
      if(list_start==NULL) 
	{
	  list_start = new linked_list;
	  assert(list_start!=NULL);
	  cur_pos = list_start;
	  cur_pos_num = 0;
	  list_start->next = NULL;
	  list_start->prior = NULL;
	}
      if(cur_pos==NULL) 
	{
	  assert(prev_pos!=NULL);
	  cur_pos = new linked_list;
	  assert(cur_pos!=NULL);
	  prev_pos->next = cur_pos;
	  cur_pos->next = NULL;
	  cur_pos->prior = prev_pos;
	}
      cur_pos->data=w_data;
    }

/* ********** bool Advance(void) *********************
   Advance the current position in the list, returning
   true if successful, else returning false.            */

  bool Advance(void)
    {
      if(cur_pos!=NULL)
	{
	  prev_pos=cur_pos;
	  cur_pos=cur_pos->next;
	  cur_pos_num++;
	  return(true);
	}
      else return(false);
    }

/* ********** bool Previous(void) *********************
   Move the current position in the list back, returning
   true if successful, else returning false.            */

  bool Previous(void)
    {
      if(prev_pos!=NULL)
	{
	  cur_pos=prev_pos;
	  cur_pos_num--;
	  prev_pos = prev_pos->prior;
	  return(true);
	}
      else return(false);
    }

/* ********** bool At_End(void) ****************
   Return true if the current position is at the
   end of the list (or the list is empty), else
   return false.                                   */

  bool At_End(void) const
    {
      return((cur_pos==NULL) ? true : false);
    }
  
/* ********** long Get_Pos(void) ****************
   Return the current list position, in terms of
   the number of elements between here and the
   start of the list.  0 = Start of the list.       */
  
  long Get_Pos(void) const
    {
      return(cur_pos_num);
    }

/* ********** bool Set_Pos(const long new_pos) ***************
   Set the current list position to new_pos items from the
   start of the list.  If the new_pos is outside the range of
   the list, Set_Pos will get as close as it can.  true is
   returned on success, else returns false.                          */

  bool Set_Pos(const long new_pos) 
    {
      long int i;
      
      if(new_pos==cur_pos_num) return(true);
      if(new_pos<0) return(false);
      if(list_start==NULL) return(false);
      if(new_pos<cur_pos_num)
	for(i=cur_pos_num; i>new_pos; --i)
	  {
	    if(Previous()==false) break;
	  }
      else
	for(i=cur_pos_num; i<new_pos; ++i)
	  {
	    if(Advance()==false) break;
	  }
      if(i==new_pos) return(true);
      else           return(false);
    }

/* ********** bool Delete(void) ***************** 
   Delete the current item in the list, returning
   true if successful, otherwise returning false.   */

  bool Delete(void)
    {
      if(cur_pos==NULL) return(false);  // Return false if list is empty, or we
                                        // are at the end of the list.
      if(prev_pos==NULL)            // Do the following if we are at the start
	{                           // of the list.
	  list_start = cur_pos->next;
	  if(list_start!=NULL)
	    list_start->prior=NULL;
	  delete cur_pos;
	  cur_pos = list_start;
	  return(true);
	}
      if(cur_pos->next==NULL)
	{
	  prev_pos->next=NULL;
	  delete cur_pos;
	  cur_pos=NULL;
	  return(true);
	}      
      prev_pos->next=cur_pos->next;         // else do this code, as 
      cur_pos->next->prior=cur_pos->prior;  // we are in the middle of
      delete cur_pos;                       // the list.
      cur_pos = prev_pos->next;
      return(true);
    }

/* ********** void Insert(eltype idata) ***************** 
   Insert the item idata in the (possibly empty) list, at
   the current list position.                              */

  void Insert(eltype idata)
    {
      if(cur_pos==NULL)                   // Use the Write function
	{                                 // if the list is empty.
	  Write(idata);
	}
      
      else if(prev_pos==NULL)             // Do the following if we
	{                                 // are at the start of the
	  list_start = new linked_list;   // list.
	  list_start->next=cur_pos;
	  cur_pos->prior=list_start;
	  list_start->prior=NULL;
	  list_start->data = idata;
	  cur_pos = list_start;
	}
      else                                  // else do this code, as
	{                                   // we are in the middle
	  assert(prev_pos->next==cur_pos);  // of the list.
	  prev_pos->next = new linked_list;
	  cur_pos->prior = prev_pos->next;
	  prev_pos->next->data = idata;
	  prev_pos->next->next = cur_pos;
	  prev_pos->next->prior = prev_pos;
	  cur_pos = prev_pos->next;
	}
    }
};

#endif
