/* This is the code for my button class */
/* (c)1995 Alexis Ashley             milamber@dcs.warwick.ac.uk */
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "button.H"
#include <assert.h>
#include <string.h>

char Button::Foreground[80]  = "Black";  // Default font used to text buttons
char Button::Background[80]  = "Grey75"; // background col used by all btns
char Button::high_name[80]   = "grey90"; // Name of colour used for bright line
char Button::low_name[80]    = "grey56"; // Name of colour used for dark line
char Button::DefaultFont[80] = "*-Courier-*-180-*";
                                  // Default font used for text buttons
UI_Globals::opt_list Button::resources[] =
    {                                // The resource list used by this class
      { "button.foreground" , Foreground  },
      { "button.background" , Background  },
      { "button.highlight"  , high_name   },
      { "button.lowlight"   , low_name    },
      { "button.font"       , DefaultFont },
      { ""                  , NULL        }
    };

const unsigned int Button::BorderWidth = 1;       // border with of all buttons
XFontStruct *Button::Default_Font = NULL;  // used in loading default font
GC Button::highlight;          // GC of bright line used to make boxes look 3D
GC Button::lowlight;           // GC of dark line used to make boxes look 3D
colour Button::high_colour;    // Stores colour for bright line
colour Button::low_colour;     // Stores colour for dark line
int Button::count=0;           // Tells the constructor that this is the first
                               // button, and so to allocate the colours for
                               // the 3D lines

XGCValues Button::Default_xGCv = 
{
  0,              // Function        - Not set
  0,              // Plane mask      - Not set
  0,              // Fg pixel col    - Not set
  0,              // Bg pixel col    - Not set
  1,              // Line width
  LineSolid,      // Line style
  CapButt,        // Cap style
  JoinMiter,      // Join style
  FillSolid       // Fill style
  };

unsigned long Button::Default_gc_mask = GCLineWidth | GCLineStyle | GCCapStyle
                                         | GCJoinStyle | GCFillStyle;

/* *********************** Text Button Constructor ******************* */

Button::Button(UI_Globals *parent,world_c x, world_c y,
	       const char * const label,status (*call_back)(byte *),
	       byte * const data, corner relative_to,
	       const char *fgcolour,char *font)
: UI_Globals(parent)
{
  unsigned int px,py;                      // Used in calculating x,y pos for button
  unsigned long mask=0;             // This is used as the mask for xgcv
  XGCValues xgcv;                   // Used to set the values to pass to
                                    // XCreateGC
  XColor tmp_col;                   // Used as a temp colour store
  unsigned int height, width;              // The height & width of this button
  Window win_id;                    // The window id of this window

// Check the X resources
  CheckResources(resources);

// If the user didn't specify a font name, use Default_Font.  If they did,
// load this font into memory.  If this fails, use Default_Font.

  if(strcmp(DefaultFont,font)==0)
    Font = Default_Font;
  else
    {
      Font = XLoadQueryFont(DispPointer(),font);
      if(Font==NULL)
	Font = Default_Font;
    }

// If necessary, load DefaultFont into memory

  if(Font==NULL && Default_Font==NULL)
    {
      Font = Default_Font = XLoadQueryFont(DispPointer(),DefaultFont);
    }
  assert(Font != NULL);

// Copy the data from the constructor's parameters into the class' private vars

  assert(label != NULL);
  btn_label = new char[strlen(label)+1];
  assert(btn_label != NULL);
  strcpy(btn_label,label);
  width=XTextWidth(Font,label,strlen(label))+x_spacing()*2;
  height=Font->max_bounds.ascent + Font->max_bounds.descent + y_spacing();
  callback = call_back;
  callback_data = data;

// Now work out where to place the button, by using the parent window's
// world coord to pixel functions

  px = parent->wc2pix_x(x);
  py = parent->wc2pix_y(y);

// This switch is used to set which corner the x and y world coords referred
// to, and to adjust px & py accordingly, so that they are correct.

  switch(relative_to)
    {
    case topleft: 
      break;
    case topright:
      px -= width;
      break;
    case bottomleft:
      py -= height;
      break;
    case bottomright:
      px -= width;
      py -= height;
      break;
    }

  x_pos = parent->pix_x2wc(px);
  y_pos = parent->pix_y2wc(py);

// The following is just some fairly bog standard code to ask X for two colour
// values.  If X is unable to comply, the code will use black & white, which
// is always guarenteed to be available.

  if(XParseColor(DispPointer(),Colourmap,Background,&tmp_col)==0 ||
     XAllocColor(DispPointer(),Colourmap,&tmp_col)==0)
     bg_colour = White;
  else
     bg_colour = tmp_col.pixel;
  if(XParseColor(DispPointer(),Colourmap,fgcolour,&tmp_col)==0 ||
     XAllocColor(DispPointer(),Colourmap,&tmp_col)==0)
     fg_colour = Black;
  else
     fg_colour = tmp_col.pixel;

// Now create the window

  win_id=XCreateSimpleWindow(DispPointer(),parent->WinId(),px,py,width,height,
			     BorderWidth,Black,bg_colour);

// Now 'register' with UI_Globals the window, so that it can work out
// world coords correctly inside my window.  This will also create the
// structures to allow this window to receive events.

  RegisterWindow(this,win_id,height,width);

// Now create a graphics context, so that I can (later on) draw inside the
// window I've just created.

  xgcv = Default_xGCv;
  xgcv.foreground = fg_colour;
  xgcv.background = bg_colour;
  xgcv.font = Font->fid;
  mask = Default_gc_mask | GCForeground | GCBackground | GCFont;
  GraphicsContext = XCreateGC(DispPointer(),win_id,mask,&xgcv);

// If it's the first initiation of the class, then the colours for the bright
// and dark lines need to be allocated & the GCs need creating

  if(count++==0)
    {
      if(XParseColor(DispPointer(),Colourmap,high_name,&tmp_col)==0 ||
	 XAllocColor(DispPointer(),Colourmap,&tmp_col)==0)
	high_colour = White;
      else
	high_colour = tmp_col.pixel;
      if(XParseColor(DispPointer(),Colourmap,low_name,&tmp_col)==0 ||
	 XAllocColor(DispPointer(),Colourmap,&tmp_col)==0)
	low_colour = Black;
      else
	low_colour = tmp_col.pixel;
      mask = Default_gc_mask | GCForeground | GCBackground;
      xgcv = Default_xGCv;
      xgcv.background = bg_colour;
      xgcv.foreground = high_colour;
      highlight = XCreateGC(DispPointer(),win_id,mask,&xgcv);
      xgcv.foreground = low_colour;
      lowlight = XCreateGC(DispPointer(),win_id,mask,&xgcv);
    }

// Now calculate where the bright & dark lines have to be draw to make the
// button look 3D

  high_seg[0].x1=0;                    high_seg[0].y1=0;
  high_seg[0].x2=width;                high_seg[0].y2=0;
  high_seg[1].x1=1;                    high_seg[1].y1=1;
  high_seg[1].x2=width-1;              high_seg[1].y2=1;
  high_seg[2].x1=0;                    high_seg[2].y1=1;
  high_seg[2].x2=0;                    high_seg[2].y2=height;
  high_seg[3].x1=1;                    high_seg[3].y1=2;
  high_seg[3].x2=1;                    high_seg[3].y2=height-1;

  low_seg[0].x1=1;                     low_seg[0].y1=height-BorderWidth;
  low_seg[0].x2=width;                 low_seg[0].y2=height-BorderWidth;
  low_seg[1].x1=2;                     low_seg[1].y1=height-1-BorderWidth;
  low_seg[1].x2=width-1;               low_seg[1].y2=height-1-BorderWidth;
  low_seg[2].x1=width-BorderWidth;     low_seg[2].y1=1;
  low_seg[2].x2=width-BorderWidth;     low_seg[2].y2=height-1;
  low_seg[3].x1=width-1-BorderWidth;   low_seg[3].y1=2;
  low_seg[3].x2=width-1-BorderWidth;   low_seg[3].y2=height-2;

// Now I tell X what events I want to be told about
  XSelectInput(DispPointer(),win_id,ExposureMask|ButtonPressMask|
	       ButtonReleaseMask);
// Finally, map the window
  XMapWindow(DispPointer(),win_id);
}

Button::~Button(void)
{
  colour cols[4];
  int num_cols=2;

// Free the font, if one was loaded specifically for this button

  if(Font!=Default_Font)
    XFreeFont(DispPointer(),Font);

// Free the colours that were allocated by the button
  cols[0] = fg_colour;
  cols[1] = bg_colour;
  if(--count==0)               // If this is the last button, unallocate the
    {                          // colours used for the 3D lines, and unload
      cols[2] = high_colour;   // the font, if neccessary
      cols[3] = low_colour;
      num_cols = 4;
      if(Default_Font!=NULL)
	{
	  XFreeFont(DispPointer(),Default_Font);
	  Default_Font = NULL;
	}
    }
  for(int i=0; i<num_cols; i++)
    if(cols[i]!=White && cols[i]!=Black)
      XFreeColors(DispPointer(),Colourmap,&cols[i],1,0);

/* Destroy the graphics context that was created for this window */
  XFreeGC(DispPointer(),GraphicsContext);

/* Destory all the data that was allocated in the constructor function */
  delete btn_label;
}

Button::status Button::event_handler(XEvent *Event)
{
  status ret_val=okay;

  switch(Event->type)
    {
    case Expose:
      if(Event->xexpose.count==0)
	{
	  XClearWindow(DispPointer(),WinId());
	  XDrawSegments(DispPointer(),WinId(),highlight,high_seg,4);
	  XDrawSegments(DispPointer(),WinId(),lowlight,low_seg,4);
	  XDrawString(DispPointer(),WinId(),GraphicsContext,x_spacing(),
		      height()-y_spacing()-BorderWidth,btn_label,
		      strlen(btn_label));
	}
      break;
    case ButtonPress:
      XClearWindow(DispPointer(),WinId());
      XDrawSegments(DispPointer(),WinId(),highlight,low_seg,4);
      XDrawSegments(DispPointer(),WinId(),lowlight,high_seg,4);
      XDrawString(DispPointer(),WinId(),GraphicsContext,x_spacing()+1,
		  height()-y_spacing()+1,btn_label,
		  strlen(btn_label));
      break;
    case ButtonRelease:
      XClearWindow(DispPointer(),WinId());
      XDrawSegments(DispPointer(),WinId(),highlight,high_seg,4);
      XDrawSegments(DispPointer(),WinId(),lowlight,low_seg,4);
      XDrawString(DispPointer(),WinId(),GraphicsContext,x_spacing(),
		  height()-y_spacing()-BorderWidth,btn_label,
		  strlen(btn_label));
      XFlush(DispPointer());
      ret_val = (*(callback))(callback_data);
      break;
    }
  return(ret_val);
}

world_c Button::X_Pos(void)
{
  return(x_pos);
}

world_c Button::Y_Pos(void)
{
  return(y_pos);
}

void Button::MoveWindow(world_c x, world_c y)
{
  x_pos = x;
  y_pos = y;
  UI_Globals *parent = Parent();
  unsigned int px = parent->wc2pix_x(x);
  unsigned int py = parent->wc2pix_y(y);
  XMoveWindow(DispPointer(),WinId(),px,py);
}
