/* The header file for my C++ button class */
/* (C)1994 Alexis 'Milamber' Ashley */
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __BUTTON__
#define __BUTTON__

#include "global.H"

class Button : public UI_Globals
{
private:
  static char Foreground[80];     // Foreground colour used for all buttons
  static char DefaultFont[80];    // Name of the default button font
public:
  Button(UI_Globals *parent, world_c x, world_c y, const char * const label,
	 status (*call_back)(byte *), byte * const data=NULL,
	 corner relative_to=topleft,const char *fgcolour=Foreground,
	 char *font=DefaultFont);
  ~Button(void);
  world_c X_Pos(void);
  world_c Y_Pos(void);
  void MoveWindow(world_c x,world_c y);
private:
// *** Private vars ***
  status (*callback)(byte *);  // This points to the callback function
  byte *callback_data;         // This points to the callback's data
  char *btn_label;             // This stores the text for the text button
  GC GraphicsContext;          // The button's text graphics context
  colour fg_colour;            // The button's foreground text colour
  colour bg_colour;            // The button's background text colour
  XFontStruct *Font;           // The button's font
  XSegment high_seg[4];        // Stores coords of the bright lines to draw
  XSegment low_seg[4];         // Stores coords of the dark lines to draw
  world_c x_pos;               // Last known x pos of the button
  world_c y_pos;               // Last known y pos of the button
// *** Private static vars ***
  static int count;            // The count of the number of buttons created
  static char Background[80];  // Background colour used for all buttons
  static char high_name[80];   // Name of colour used for bright 3D line
  static char low_name[80];    // Name of colour used for dark 3D line
  static opt_list resources[]; // The resources used by buttons
  static const unsigned int BorderWidth;  // Borderwidth of all buttons
  static GC highlight;         // GC of bright line used to make boxes look 3D
  static GC lowlight;          // GC of dark line used to make boxes look 3D
  static colour high_colour;   // Stores colour for bright line
  static colour low_colour;    // Stores colour for dark line
  static XFontStruct *Default_Font; // Default font used for buttons
  static XGCValues Default_xGCv;    // Default values to use when making GCs
  static unsigned long Default_gc_mask; // The GC mask for Default_xGCv
// *** Private functions ***
  status event_handler(XEvent *);  // This fn is called by UI_Globals
};

#endif  




