/****************************************************************/
/*  This header file (and its associated C file) handle all the */
/*  dialogues for the user interface.                           */
/*    (c)1995 Alexis Ashley          milamber@dcs.warwick.ac.uk */
/****************************************************************/

/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __DIALOGUES__
#define __DIALOGUES__

#include "main_win.H"
#include "../globals/list.H"

void notice(MainWindow *,char *);  // Gives the user the message pointed to by
                                   // the char * pointer, and an okay 
                                // button. It then waits until they press okay

bool question(MainWindow *,char *);  // Gives the user the message pointed to
                                     // by the char * pointer, and yes and no
                                     // buttons. It then waits until they
                                     // select the yes or no button.  Yes
                                     // will return true, else returns false

bool confirm(MainWindow *,char *);   // Gives the user the message pointed to
                                     // by the char * pointer, and yes and no
                                     // buttons. It then waits until they
                                     // select the yes or no button.  Yes
                                     // will return true, else returns false.
                                     // The difference between this and a
                                     // question button is that it also has
                                     // a "don't ask again" button, which
                                     // will cause all subsequent calls to
                                     // return yes (ie true).

char *load_file(MainWindow *,char *title);
                                       // Creates a file selection dialogue
                                       // box.  It will return the
                                       // filename they have selected. NULL
                                       // indicates they selected CANCEL

char *save_file(MainWindow *,char *title);
                                       // Creates a file selection dialogue
                                       // box.  It allows a new filename
                                       // to be entered.  It will return the
                                       // filename they have selected. NULL
                                       // indicates they selected CANCEL

char *typein_text(MainWindow *,char *title);
                                       // Creates a type-in dialogue
                                       // box.  It allows a string
                                       // to be entered.  It will return the
                                       // string the user typed in. NULL
                                       // indicates they selected CANCEL

char *selection_box(MainWindow *mw,char *title, List <char *> *selections);
                                       // Creates a selection dialogue box.
                                       // Allows the user to select one item
                                       // from selections

float select_number(MainWindow *mw,char *title, float start, float minval,
		    float maxval, float increment);
		                       // Allows the user to select a number,
                                       // between start and end inclusively,
                                       // in steps of increment.  The
                                       // function will return the answer, or
                                       // start-1 if user selects cancel

#endif

