/* This is the header file for the top level window interface class UI */
/* (c)1994 Alexis 'Milamber' Ashley */
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __MAIN_WIN__
#define __MAIN_WIN__

#include "global.H"

class MainWindow : public UI_Globals
{
public:
  MainWindow(const char * const name, unsigned int height, unsigned int width, 
	     int argc, char **argv);
  ~MainWindow(void);
  void BusyCursor(void);       // Show the 'busy' cursor
  void NormalCursor(void);     // Show the normal cursor
  void ResizeWindow(unsigned int new_height,unsigned int new_width); // Resizes the window
  void RegisterConfigureFunction(void (*callback)(void));
                                  // This registers a function that will be
                                  // called when the main window is
                                  // reconfigured.
  Window SetFocus(Window);     // Sets the window that wants keyboard focus +
                               // returns the old focus window
  int ColourDepth(void);       // Returns colourdepth of window
  unsigned int ScreenHeight(void);    // Returns height of screen
  unsigned int ScreenWidth(void);     // Returns width of screen
  void SetName(char *name);       // Sets the title of the main window
private:
  status event_handler(XEvent *); // The event handler for this window
  colour fg_colour;           // The foreground colour used for this window
  colour bg_colour;           // The background colour used for this window
  XColor c_fg_colour;         // The foreground colour used for the cursor
  XColor c_bg_colour;         // The background colour used for the cursor
  static char Foreground[80]; // Foreground colour for the main window
  static char Background[80]; // Background colour for the main window
  static char CursorColour[80];// Foreground colour for the cursor
  static unsigned int BorderWidth;   // Width of main window's border
  unsigned int main_x;               // X pixel position of the main window
  unsigned int main_y;               // Y pixel position of the main window
  int main_gravity;           // Gravity of this window
  Cursor arrow;               // normal (arrow) cursor
  Cursor sands;               // the 'sands of time' cursor
  Pixmap sands_bmap;          // the "sands of time" cursor's bitmap
  Pixmap sands_bmask;         // the "sands of time" cursor's mask
  void (*configure_fn)(void); // Pointer to the configure function
  Window FocusWindow;         // the current window that wants keyboard focus
};

#endif
