/*****************************************************************************/
/* This is the header file for the simple sub window class.  It provides the */
/* bear minimum functions for a sub window.  It is designed to be inherited  */
/* by other classes, which build on the functions provided here.             */
/*                                                                           */
/*                                (c)1994 Alexis 'Milamber' Ashley           */
/*****************************************************************************/
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __SIMPLE_SUB_WIN__
#define __SIMPLE_SUB_WIN__

#include "global.H"

class SimpleSubWindow : public UI_Globals
{
protected:
  static char Foreground[80];           // Foreground colour for sub windows
  static char Background[80];           // Background colour for sub windows
  static unsigned int BorderWidth;      // Border width for the sub window
public:
  SimpleSubWindow(UI_Globals *parent, unsigned int x, unsigned int y,
		  unsigned int height, unsigned int width,
		  UI_Globals *class_ptr,long event_mask=0, 
		  const char * const fgcolour = Foreground,
		  const char * const bgcolour = Background,
		  unsigned int Border_Width=BorderWidth);

  SimpleSubWindow(UI_Globals *parent, world_c x, world_c y,
		  unsigned int height, unsigned int width,
		  UI_Globals *class_ptr,long event_mask=0,
		  corner relative_to=topleft,
		  const char * const fgcolour = Foreground,
		  const char * const bgcolour = Background,
		  unsigned int Border_Width=BorderWidth);

  SimpleSubWindow(UI_Globals *parent, world_c x, world_c y, world_c height,
		  world_c width,UI_Globals *class_ptr,long event_mask=0,
		  const char * const fgcolour = Foreground,
		  const char * const bgcolour = Background,
		  unsigned int Border_Width=BorderWidth);

  ~SimpleSubWindow(void);
  world_c X_Pos(void);     // Returns the X pos of the top left corner
  world_c Y_Pos(void);     // Returns the Y pos of the top left corner
  void MoveWindow(world_c x,world_c y);  // Moves window to position x,y
  void ResizeWindow(world_c new_height,world_c new_width);
                           // Changes window's size to new values
  void CenterWindow(void); // Moves window to middle of parent window
protected:
  colour fg_colour;        // The foreground colour of this window
  colour bg_colour;        // The background colour of this window
private:
  void Constructor(UI_Globals *parent, unsigned int x, unsigned int y,
		   unsigned int height, unsigned int width,
		   UI_Globals *class_ptr,long event_mask, 
		   const char * const fgcolour,const char * const bgcolour,
		   unsigned int Border_Width);
  world_c x_pos;    // Last known x pos of window
  world_c y_pos;    // Last known y pos of window
};

#endif
