/* This is the header file for text windows of my user interface class UI */
/* (c)1994 Alexis 'Milamber' Ashley */
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __TEXT_WIN__
#define __TEXT_WIN__

#include "sub_win.H"
#include "list.H"

#include <sys/time.h>
#include <unistd.h>

class TextWindow : public SimpleSubWindow
{
protected:
  static char DefaultFont[80];   // Default font used for sub windows
  static unsigned int BorderWidth;      // Size of border 
public:
  TextWindow(UI_Globals *parent,world_c x,world_c y,world_c height,
	     world_c width, bool selectable=false,
	     const char * const fgcolour = Foreground,
	     const char * const bgcolour = Background,
	     const char * const font = DefaultFont,
	     unsigned int Border_Width=BorderWidth);
  ~TextWindow(void);
  long int AddText(world_c x, world_c y,const char *const str,
		   const char * const fgcolour = Foreground);
  void DeleteText(long int pos);
  void MoveText(long int pos, world_c x, world_c y);
  void ClearAllText(void);
  char *CurrentSelection(void);
  world_c TextWidth(UI_Globals *relative_to,char *);
  world_c TextHeight(UI_Globals *relative_to);
  void ResizeWindow(world_c new_height, world_c new_width);
  void RegisterCallback(status (*callback)(byte *), byte *callback_data=NULL);
private:
// **** Private Structures ****
struct Text
{
  unsigned int x,y;
  char *string;
  GC gc;
  bool inverse;
};
// *** Private vars ***
  GC GraphicsContext;      // The GC for the sub window
  char *fg_col;            // The name of fg_colour
  List <Text> TList;       // This list holds the text to draw
  XFontStruct *xfs;        // The font for this subwindow
  XSegment high_seg[4];    // Stores coords of the bright lines to draw
  XSegment low_seg[4];     // Stores coords of the dark lines to draw
  int SendEventFlag;       // Used to stop multiple send events being made
  GC InverseGC;            // The GC for selected text
  bool selectable;         // True if text can be selected with the mouse
  status (*callback)(byte *);  // This points to the callback function
  byte *callback_data;         // This points to the callback's data
  timeval last_click;          // This holds the time of the last mouse click
// *** Static private vars
  static XFontStruct *Default_Font; // Default font for all text windows
  static char high_name[80];        // Default colour for bright 3D lines
  static char low_name[80];         // Default colour for dark 3D lines
  static char click_speed[80];      // Default min time between double clicks
  static opt_list resources[];      // Resource list for all text windows
  static colour high_colour;        // Colour value for bright lines
  static colour low_colour;         // Colour value for dark lines
  static GC highlight;              // GC for bright lines
  static GC lowlight;               // GC for dark lines
  static int count;                 // Count of how many text windows there are
// *** Private functions
  status event_handler(XEvent *);  // The subwindow's event handler
};

#endif
