/* The header file for my C++ TypeInBox class */
/* (C)1994 Alexis 'Milamber' Ashley */
/*
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __TYPEINBOX__
#define __TYPEINBOX__

#include "main_win.H"

class TypeInBox : public UI_Globals
{
private:
  static char Foreground[80];     // Foreground colour used for all TypeInBoxes
  static char DefaultFont[80];    // Name of the default TypeInBox font
public:
  TypeInBox(UI_Globals *parent, world_c x, world_c y, MainWindow *mw,
	    status (*call_back)(char *),int size=80, 
	    corner relative_to=topleft,
	    const char *fgcolour=Foreground, char *font=DefaultFont);
  ~TypeInBox(void);
  char *CurrentValue(void);      // Returns the current text. You must do a 
                                 // delete to nuke the text that is returned
  void ClearText(void);          // Wipes the current text
  void SetText(char *new_text);  // Sets the current text to new_text
  void MoveWindow(world_c new_x,world_c new_y); //Moves the window
private:
// *** Private vars ***
  status (*callback)(char *);  // This points to the callback function
  int buf_size;                // Size of char_buf
  int cur_pos;                 // Current cursor pos in char_buf
  char *char_buf;              // This stores the text that has been typed in
  XFontStruct *Font;           // The TypeInBox's font
  GC GraphicsContext;          // The TypeInBox's text graphics context
  colour fg_colour;            // The TypeInBox's foreground text colour
  colour bg_colour;            // The TypeInBox's background text colour
  XSegment high_seg[4];        // Stores coords of the bright lines to draw
  XSegment low_seg[4];         // Stores coords of the dark lines to draw
  char xlat_buffer[20];        // Used for translating the gibberish that X
                               // gives as a keyboard entry to something that
                               // I can use
  MainWindow *MainWin;         // This is used to set & restore keyboard focus
  Window old_focus;            // The old focus window
// *** Private static vars ***
  static int count;            // The count of the number of type in boxes
  static char Background[80];  // Background colour used for all TypeInBoxs
  static char high_name[80];   // Name of colour used for bright 3D line
  static char low_name[80];    // Name of colour used for dark 3D line
  static opt_list resources[]; // The resources used by TypeInBoxs
  static const unsigned int BorderWidth;  // Borderwidth of all TypeInBoxs
  static GC highlight;         // GC of bright line used to make boxes look 3D
  static GC lowlight;          // GC of dark line used to make boxes look 3D
  static colour high_colour;   // Stores colour for bright line
  static colour low_colour;    // Stores colour for dark line
  static XFontStruct *Default_Font; // Default font used for TypeInBoxs
  static XGCValues Default_xGCv;    // Default values to use when making GCs
  static unsigned long Default_gc_mask; // The GC mask for Default_xGCv
// *** The event handler ***
  status event_handler(XEvent *);  // This fn is called by UI_Globals
};

#endif  




